#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'

from persistent import Persistent
from zope.interface import Interface, implementer
from zope.location import Location
from zope.schema.fieldproperty import FieldProperty

from pyams_content.component.illustration import IBasicIllustrationTarget, ILinkIllustration
from pyams_content.component.illustration.interfaces import IIllustration
from pyams_content.component.links import IInternalLink
from pyams_content.features.renderer.interfaces import IContentRenderer
from pyams_content.features.renderer.skin import BaseContentRenderer
from pyams_content.interfaces import IBaseContent
from pyams_content.root import ISiteRoot
from pyams_default_theme import _
from pyams_default_theme.component.illustration.interfaces import IIllustrationRenderer, IIllustrationWithZoomSettings, \
    ILLUSTRATION_AFTER_BODY, ILLUSTRATION_BEFORE_BODY
from pyams_default_theme.interfaces import IContentHeaderIllustration, IContentNavigationIllustration
from pyams_skin.layer import IPyAMSLayer
from pyams_template.template import template_config
from pyams_utils.adapter import ContextRequestViewAdapter, adapter_config, get_annotation_adapter
from pyams_utils.interfaces.tales import ITALESExtension


#
# Illustrations adapters
#

@adapter_config(context=(IBasicIllustrationTarget, IPyAMSLayer), provides=IContentNavigationIllustration)
def base_content_navigation_illustration_adapter(context, request):
    """Default content navigation illustration adapter"""
    illustration = ILinkIllustration(context, None)
    if not (illustration and illustration.has_data()):
        illustration = IIllustration(context, None)
    if illustration and illustration.has_data():
        return illustration
    if IInternalLink.providedBy(context):
        target = context.get_target()
        if target is not None:
            illustration = request.registry.queryMultiAdapter((target, request),
                                                              IContentNavigationIllustration)
            if illustration and illustration.has_data():
                return illustration


@adapter_config(name='pyams_illustration',
                context=(Interface, Interface, Interface),
                provides=ITALESExtension)
class PyAMSContentIllustrationTALESExtension(ContextRequestViewAdapter):
    """PyAMS content illustration TALES extension"""

    def render(self, context=None):
        if context is None:
            context = self.context
        return self.request.registry.queryMultiAdapter((context, self.request), IContentNavigationIllustration)


@adapter_config(context=(ISiteRoot, IPyAMSLayer), provides=IContentHeaderIllustration)
@adapter_config(context=(IBaseContent, IPyAMSLayer), provides=IContentHeaderIllustration)
def base_content_header_illustration_adapter(context, request):
    """Base content header illustration adapter"""
    illustration = IIllustration(context, None)
    if illustration and illustration.has_data():
        return illustration


#
# Illustration renderer with zoom settings
#

ILLUSTRATION_ZOOM_RENDERER_SETTINGS_KEY = 'pyams_content.illustration.renderer:zoom'


@implementer(IIllustrationWithZoomSettings)
class IllustrationZoomSettings(Persistent, Location):
    """Illustration zoom renderer settings"""

    zoom_on_click = FieldProperty(IIllustrationWithZoomSettings['zoom_on_click'])


@adapter_config(context=IIllustration, provides=IIllustrationWithZoomSettings)
def illustration_with_zoom_settings_factory(context):
    """Illustration zoom renderer settings factory"""
    return get_annotation_adapter(context, ILLUSTRATION_ZOOM_RENDERER_SETTINGS_KEY, IllustrationZoomSettings)


#
# Illustration renderers
#

@implementer(IIllustrationRenderer)
class BaseIllustrationRenderer(BaseContentRenderer):
    """Base illustration renderer"""

    context_attrs = ('author',)
    i18n_context_attrs = ('title', 'alt_title', 'description', 'data')

    position = None


@adapter_config(name='centered-before-body', context=(IIllustration, IPyAMSLayer), provides=IContentRenderer)
@template_config(template='templates/illustration-default.pt', layer=IPyAMSLayer)
class DefaultIllustrationRenderer(BaseIllustrationRenderer):
    """Default illustration renderer"""

    label = _("Centered illustration before text")
    weight = 1

    position = ILLUSTRATION_BEFORE_BODY


@adapter_config(name='left+zoom', context=(IIllustration, IPyAMSLayer), provides=IContentRenderer)
@template_config(template='templates/illustration-left.pt', layer=IPyAMSLayer)
class LeftIllustrationWithZoomRenderer(BaseIllustrationRenderer):
    """Illustration renderer with small image and zoom"""

    label = _("Small illustration on the left")
    weight = 2

    position = ILLUSTRATION_BEFORE_BODY
    settings_interface = IIllustrationWithZoomSettings


@adapter_config(name='right+zoom', context=(IIllustration, IPyAMSLayer), provides=IContentRenderer)
@template_config(template='templates/illustration-right.pt', layer=IPyAMSLayer)
class RightIllustrationWithZoomRenderer(BaseIllustrationRenderer):
    """Illustration renderer with small image and zoom"""

    label = _("Small illustration on the right")
    weight = 3

    position = ILLUSTRATION_BEFORE_BODY
    settings_interface = IIllustrationWithZoomSettings


@adapter_config(name='default', context=(IIllustration, IPyAMSLayer), provides=IContentRenderer)
@template_config(template='templates/illustration-default.pt', layer=IPyAMSLayer)
class DefaultIllustrationRenderer(BaseIllustrationRenderer):
    """Default illustration renderer"""

    label = _("Centered illustration after text")
    weight = 4

    position = ILLUSTRATION_AFTER_BODY
